import sys
import os

from oslo_config import cfg
from oslo_log import log as logging
from oslo_service import service

from magellan.api import service as api_service
from magellan.common import config
from magellan.db import api as db_api
from magellan import version
from magellan.engine import scheduler


# If ../magellan/__init__.py exists, add ../ to Python search path, so that
# it will override what happens to be installed in /usr/(local/)lib/python...
POSSIBLE_TOPDIR = os.path.normpath(os.path.join(os.path.abspath(sys.argv[0]),
                                                os.pardir,
                                                os.pardir))
if os.path.exists(os.path.join(POSSIBLE_TOPDIR, 'magellan', '__init__.py')):
    sys.path.insert(0, POSSIBLE_TOPDIR)


CONF = cfg.CONF

launcher = service.ServiceLauncher(CONF)


def launch_api():
    launcher.launch_service(api_service.WSGIService('magellan_api'))


def launch_scheduler():
    launcher.launch_service(scheduler.JobScheduler())


def launch_any(options):
    if CONF.db_sync:
        db_api.db_sync(db_api.get_engine())
        logging.getLogger(__name__).info("Magellan DB synced")
    db_api.execution_cleanup()
    try:
        for option in options:
            LAUNCH_OPTIONS[option]()

        launcher.wait()
    except RuntimeError as e:
        sys.stderr.write("ERROR: %s\n" % e)
        sys.exit(1)


# Map cli options to appropriate functions. The cli options are
# registered in magellan's config.py.
LAUNCH_OPTIONS = {
    'api': launch_api,
    'scheduler': launch_scheduler,
}

MAGELLAN_TITLE = str("Starting Magellan Autodiscovery Service, version %s"
                     % version.version_info.version_string())


def print_server_info():
    print(MAGELLAN_TITLE)

    comp_str = ("[%s]" % ','.join(LAUNCH_OPTIONS)
                if cfg.CONF.server == ['all'] else cfg.CONF.server)

    print('Launching server components %s...' % comp_str)


def get_properly_ordered_parameters():
    """Orders launch parameters in the right order.
    In oslo it's important the order of the launch parameters.
    if --config-file came after the command line parameters the command
    line parameters are ignored.
    So to make user command line parameters are never ignored this method
    moves --config-file to be always first.
    """
    args = sys.argv[1:]

    for arg in sys.argv[1:]:
        if arg == '--config-file' or arg.startswith('--config-file='):
            if "=" in arg:
                conf_file_value = arg.split("=", 1)[1]
            else:
                conf_file_value = args[args.index(arg) + 1]
                args.remove(conf_file_value)
            args.remove(arg)
            args.insert(0, "--config-file")
            args.insert(1, conf_file_value)

    return args


def main():
    try:
        config.parse_args(get_properly_ordered_parameters())
        print_server_info()

        logging.setup(CONF, 'Magellan')

        if cfg.CONF.server == ['all']:
            # Launch all servers.
            launch_any(LAUNCH_OPTIONS.keys())
        else:
            # Validate launch option.
            if set(cfg.CONF.server) - set(LAUNCH_OPTIONS.keys()):
                raise Exception(
                    'Valid options are all or any combination of ' +
                    ', '.join(LAUNCH_OPTIONS.keys()))

            # Launch distinct set of server(s).
            launch_any(set(cfg.CONF.server))

    except RuntimeError as excp:
        sys.stderr.write("ERROR: %s\n" % excp)
        sys.exit(1)


if __name__ == '__main__':
    sys.exit(main())
